package com.jeecms.cms.manager.assist.impl;

import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.jeecms.cms.dao.assist.CmsSchedulerDao;
import com.jeecms.cms.entity.assist.CmsAcquisition;
import com.jeecms.cms.entity.assist.CmsScheduler;
import com.jeecms.cms.manager.assist.CmsSchedulerMng;
import com.jeecms.common.hibernate3.Updater;
/**
 * 计划任务管理服务接口实现类
 * @author javacoo
 * @since 2011-11-07
 * @version 1.0 
 */
@Service
@Transactional
public class CmsSchedulerMngImpl implements CmsSchedulerMng{
	@Transactional(readOnly = true)
	public List<CmsScheduler> getList() {
		return dao.getList();
	}
	
	@Transactional(readOnly = true)
	public List<CmsScheduler> getListBy(CmsScheduler bean) {
		return dao.getListBy(bean);
	}

	@Transactional(readOnly = true)
	public CmsScheduler findById(Integer id) {
		CmsScheduler entity = dao.findById(id);
		return entity;
	}

	public void stop(Integer id) {
		CmsScheduler acqu = findById(id);
		if (acqu == null) {
			return;
		}
		if (acqu.getStatus() == CmsScheduler.START) {
			acqu.setStatus(CmsScheduler.STOP);
		}
	}

	
	public CmsScheduler start(Integer id) {
		CmsScheduler scheduler = findById(id);
		if (scheduler == null) {
			return scheduler;
		}
		scheduler.setStatus(CmsAcquisition.START);
		scheduler.setStartTime(new Date());
		scheduler.setEndTime(null);
		return scheduler;
	}

	public void end(Integer id) {
		CmsScheduler scheduler = findById(id);
		if (scheduler == null) {
			return;
		}
		scheduler.setStatus(CmsAcquisition.STOP);
		scheduler.setEndTime(new Date());
	}


	public CmsScheduler save(CmsScheduler bean) {
		bean.init();
		dao.save(bean);
		return bean;
	}

	public CmsScheduler update(CmsScheduler bean) {
		Updater<CmsScheduler> updater = new Updater<CmsScheduler>(bean);
		bean = dao.updateByUpdater(updater);
		return bean;
	}

	public CmsScheduler deleteById(Integer id) {
		CmsScheduler bean = dao.deleteById(id);
		return bean;
	}

	public CmsScheduler[] deleteByIds(Integer[] ids) {
		CmsScheduler[] beans = new CmsScheduler[ids.length];
		for (int i = 0, len = ids.length; i < len; i++) {
			beans[i] = deleteById(ids[i]);
		}
		return beans;
	}

	private CmsSchedulerDao dao;

	

	@Autowired
	public void setDao(CmsSchedulerDao dao) {
		this.dao = dao;
	}

}