package com.jeecms.cms.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.lang.StringUtils;
import org.javacoo.crawler.CrawlController;
import org.javacoo.crawler.constants.Constants;
import org.javacoo.crawler.data.CrawlScope;
import org.javacoo.crawler.filter.BriefAreaFilter;
import org.javacoo.crawler.filter.ContentAreaFilter;
import org.javacoo.crawler.filter.FieldFilter;
import org.javacoo.crawler.filter.Filter;
import org.javacoo.crawler.filter.LinkAreaFilter;
import org.javacoo.crawler.filter.PaginationAreaFilter;
import org.javacoo.crawler.persistent.CrawlerPersistent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.jeecms.cms.entity.assist.CmsAcquisition;
import com.jeecms.cms.manager.assist.CmsAcquisitionMng;

@Service
public class AcquisitionSvcImpl implements AcquisitionSvc {
	private Logger log = LoggerFactory.getLogger(AcquisitionSvcImpl.class);
	private CmsAcquisitionMng cmsAcquisitionMng;
	/**爬虫持久化对象*/
	private CrawlerPersistent crawlerPersistent;
	
	
	/**存放CrawlController的Map对象*/
	private static ConcurrentHashMap<Integer,CrawlController> crawlControllerMap = new ConcurrentHashMap<Integer,CrawlController>();
	@Autowired
	public void setCmsAcquisitionMng(CmsAcquisitionMng cmsAcquisitionMng) {
		this.cmsAcquisitionMng = cmsAcquisitionMng;
	}
	@Autowired
	public void setCrawlerPersistent(CrawlerPersistent crawlerPersistent) {
		this.crawlerPersistent = crawlerPersistent;
	}

	public boolean start(Integer id) {
		CmsAcquisition acqu = cmsAcquisitionMng.findById(id);
		if (acqu == null) {
			return false;
		}
		CrawlController crawlController = new CrawlController();
		
		List<Filter> filters = new ArrayList<Filter>();
		filters.add(new LinkAreaFilter(acqu.getLinksetStart(),acqu.getLinksetEnd()));
		filters.add(new ContentAreaFilter(acqu.getContentStart(),acqu.getContentEnd()));
		filters.add(new BriefAreaFilter(acqu.getDescriptionStart(),acqu.getDescriptionEnd()));
		filters.add(new PaginationAreaFilter(acqu.getPaginationStart(),acqu.getPaginationEnd()));
		//添加扩展字段过滤器
		if(StringUtils.isNotEmpty(acqu.getKeywordsStart())){
			addFilter(acqu.getKeywordsStart(),filters);
		}
		CrawlScope crawlScope = new CrawlScope();
		crawlScope.setCrawlerPersistent(crawlerPersistent);
		crawlScope.setEncoding(acqu.getPageEncoding());
		crawlScope.setId(acqu.getId());
		crawlScope.setFilterList(filters);
		crawlScope.setRepairPageUrl(acqu.getLinkStart());
		crawlScope.setRepairImageUrl(acqu.getLinkEnd());
		//设置休眠时间
		crawlScope.setSleepTime(acqu.getPauseTime());
		crawlScope.setPaginationRepairUrl(acqu.getPaginationRepairUrl());
		//是否下载图片至本地
		crawlScope.setExtractContentRes(Boolean.valueOf(acqu.getTitleStart()));
		//是否去掉内容中连接
		crawlScope.setReplaceHtmlLink(Boolean.valueOf(acqu.getTitleEnd()));
		if(StringUtils.isNotBlank(acqu.getRepeatCheckType())){
			if("true".equals(StringUtils.trim(acqu.getRepeatCheckType()))){
				crawlScope.setAllowRepeat(true);
			}else{
				crawlScope.setAllowRepeat(false);
			}
		}
		if(StringUtils.isNotBlank(acqu.getUseProxy())){
			if("true".equals(StringUtils.trim(acqu.getUseProxy()))){
				crawlScope.setUseProxy(true);
			}else{
				crawlScope.setUseProxy(false);
			}
		}
		crawlScope.setProxyAddress(acqu.getProxyAddress());
		crawlScope.setProxyPort(acqu.getProxyPort());
		
		crawlScope.addSeeds(acqu.getAllPlans());
		crawlController.initialize(crawlScope);
		crawlController.start();
		
		crawlControllerMap.put(acqu.getId(), crawlController);
		return true;
	}
	private void addFilter(String jsonStr,List<Filter> filters){
		JSONArray arry = JSONArray.fromObject(jsonStr);
		String fields= "",filterStart= "",filterEnd = "";
		for(int i = 0;i < arry.size();i++){
			if(null != JSONObject.fromObject(arry.get(i)).get("fields")){
				fields = (String)JSONObject.fromObject(arry.get(i)).get("fields");
			}
			if(null != JSONObject.fromObject(arry.get(i)).get("filterStart")){
				filterStart =(String)JSONObject.fromObject(arry.get(i)).get("filterStart");
			}
            if(null != JSONObject.fromObject(arry.get(i)).get("filterEnd")){
        	   filterEnd = (String)JSONObject.fromObject(arry.get(i)).get("filterEnd");
			}
			filters.add(new FieldFilter(fields,filterStart,filterEnd));
		}
	}
	public boolean stop(Integer id) {
		CrawlController crawlController = crawlControllerMap.get(id);
		if(null != crawlController && (Constants.CRAWL_STATE_RUNNING.equals(crawlController.getState()) || Constants.CRAWL_STATE_PAUSE.equals(crawlController.getState()))){
			crawlController.shutdownNow();
		}
		return true;
	}
	public boolean pause(Integer id) {
		CrawlController crawlController = crawlControllerMap.get(id);
		if(null != crawlController && Constants.CRAWL_STATE_RUNNING.equals(crawlController.getState())){
			crawlController.pause();
		}
		return true;
	}
	public boolean resume(Integer id) {
		CrawlController crawlController = crawlControllerMap.get(id);
		if(null != crawlController && Constants.CRAWL_STATE_PAUSE.equals(crawlController.getState())){
			crawlController.resume();
		}
		return false;
	}
}
