package com.jeecms.cms.service;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import net.sf.json.JSONArray;

import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.analysis.tokenattributes.TermAttribute;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.LockObtainFailedException;
import org.apache.lucene.store.RAMDirectory;
import org.apache.lucene.util.Version;
import org.javacoo.crawler.constants.Constants;
import org.javacoo.crawler.data.ContentBean;
import org.javacoo.crawler.data.Task;
import org.javacoo.crawler.persistent.CrawlerPersistent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.wltea.analyzer.lucene.IKAnalyzer;
import org.wltea.analyzer.lucene.IKQueryParser;
import org.wltea.analyzer.lucene.IKSimilarity;

import com.jeecms.cms.entity.assist.CmsAcquisition;
import com.jeecms.cms.entity.assist.CmsAcquisitionHistory;
import com.jeecms.cms.entity.assist.CmsAcquisitionTemp;
import com.jeecms.cms.entity.assist.CmsAcquisition.AcquisitionResultType;
import com.jeecms.cms.entity.main.CmsSite;
import com.jeecms.cms.entity.main.CmsUser;
import com.jeecms.cms.manager.assist.CmsAcquisitionHistoryMng;
import com.jeecms.cms.manager.assist.CmsAcquisitionMng;
import com.jeecms.cms.manager.assist.CmsAcquisitionTempMng;
import com.jeecms.cms.manager.main.CmsUserMng;
import com.jeecms.common.page.Pagination;

/**
 * 爬虫持久层接口实现类
 * @author javacoo
 * @since 2011-11-12
 */
@Service
public class SimpleCrawlerPersistentImpl implements CrawlerPersistent{
	private Logger log = LoggerFactory.getLogger(SimpleCrawlerPersistentImpl.class);
	/**采集管理对象*/
	private CmsAcquisitionMng cmsAcquisitionMng;
	/**采集历史对象*/
	private CmsAcquisitionHistoryMng cmsAcquisitionHistoryMng;
	/**采集临时对象*/
	private CmsAcquisitionTempMng cmsAcquisitionTempMng;
	/**分词器*/
	//private Analyzer analyzer = new StandardAnalyzer(Version.LUCENE_30);
	/**分词器*/
    private Analyzer analyzer;
	/**用户对象*/
	private CmsUserMng cmsUserMng;
	/**内部用户集合*/
	private List<CmsUser> userList;
	@Autowired
	public void setCmsAcquisitionMng(CmsAcquisitionMng cmsAcquisitionMng) {
		this.cmsAcquisitionMng = cmsAcquisitionMng;
	}
	@Autowired
	public void setAnalyzer(Analyzer analyzer) {
		this.analyzer = analyzer;
	}
	@Autowired
	public void setCmsUserMng(CmsUserMng cmsUserMng) {
		this.cmsUserMng = cmsUserMng;
	}
	@Autowired
	public void setCmsAcquisitionHistoryMng(
			CmsAcquisitionHistoryMng cmsAcquisitionHistoryMng) {
		this.cmsAcquisitionHistoryMng = cmsAcquisitionHistoryMng;
	}

	@Autowired
	public void setCmsAcquisitionTempMng(
			CmsAcquisitionTempMng cmsAcquisitionTempMng) {
		this.cmsAcquisitionTempMng = cmsAcquisitionTempMng;
	}
	/**
	 * 保存内容
	 * @param task
	 */
	public synchronized void save(Task task) {
		ContentBean contentBean = task.getContentBean();
		CmsAcquisition acqu = cmsAcquisitionMng.findById(task.getContentBean().getAcquId());
		cmsAcquisitionMng.isNeedBreak(acqu.getId(),task.getPlanNum(), task.getTaskNum(),task.getCurrTaskTotalNum());
		CmsAcquisitionTemp temp = SimpleCrawlerPersistentImpl.this.newTemp(
				task.getParentUrl(), task.getUrl(), task.getCurrTaskTotalNum() - task.getTaskNum(), (float)task.getTaskNum(), (float)task.getCurrTaskTotalNum(),
				acqu.getSite());
		CmsAcquisitionHistory history = SimpleCrawlerPersistentImpl.this
				.newHistory(task.getParentUrl(), task.getUrl(), acqu);
		try {
			//标题图片
			String titleImgPath = contentBean.getTitleImg();
			//图片,不需要保存图片地址
			Map<String,String> imagesMap = (Map<String,String>)contentBean.getResMap().get(Constants.RES_IMAGES_MAP_KEY);
			String[] imagesPath = null;
			String[] imagesDesc = null;
			if(!CollectionUtils.isEmpty(imagesMap)){
//				Iterator<String> it = imagesMap.keySet().iterator();
//				if(StringUtils.isEmpty(titleImgPath)){
//					titleImgPath = it.next();
//				}
				int imagesMapSize = imagesMap.keySet().size();
				imagesPath = new String[imagesMapSize];
				imagesDesc = new String[imagesMapSize];
				int i = 0;
				for(Iterator<String> it = imagesMap.keySet().iterator();it.hasNext();){
					imagesPath[i] = it.next();
					imagesDesc[i] = "";
					i++;
				}
			}
			//附件地址
			String[] attachmentPaths = null;
			String[] attachmentNames = null; 
			String[] attachmentFilenames = null;
			Map<String,String> attachmentMap = (Map<String,String>)contentBean.getResMap().get(Constants.RES_ATTAC_MAP_KEY);
			if(!CollectionUtils.isEmpty(attachmentMap)){
				int attachmentMapSize = attachmentMap.keySet().size();
				attachmentPaths = new String[attachmentMapSize];
				attachmentNames = new String[attachmentMapSize];
				attachmentFilenames = new String[attachmentMapSize];
				int i = 0;
				String path = "";
				for(Iterator<String> it = attachmentMap.keySet().iterator();it.hasNext();){
					path = it.next();
					attachmentPaths[i] = path;
					attachmentNames[i] = FilenameUtils.getName(path);
					attachmentFilenames[i] = FilenameUtils.getName(path);
					i++;
				}
			}
			//媒体文件
			String mediaPath = null;
			String mediaType = null;
			Map<String,String> mediaMap = (Map<String,String>)contentBean.getResMap().get(Constants.RES_MEDIA_MAP_KEY);
			if(!CollectionUtils.isEmpty(mediaMap)){
				String path = "";
				for(Iterator<String> it = mediaMap.keySet().iterator();it.hasNext();){
					path = it.next();
					mediaPath = path;
					mediaType = getMediaType(path);
				}
			}
			String[] tagArr = null;
			if(StringUtils.isNotEmpty(contentBean.getTitle())){
				List<String> tagList = getTagList(contentBean.getTitle());
				//tag标签
				tagArr = new String[tagList.size()];
				for(int i =0;i<tagList.size();i++){
					tagArr[i] = tagList.get(i);
				}
			}
			//组装内容扩展字段值
			Map<String, String> attr = new HashMap<String, String>();
			String key = null;
			List<String> values = null;
			for(Iterator<String> it = contentBean.getFieldValueMap().keySet().iterator();it.hasNext();){
				key = it.next();
				values = contentBean.getFieldValueMap().get(key);
				if(!CollectionUtils.isEmpty(values) && values.size() == 1){
					attr.put(key, values.get(0));
				}else if(!CollectionUtils.isEmpty(values) && values.size() > 1){
					//以JSON格式存储
					attr.put(key, JSONArray.fromObject(values).toString());
				}
			}
			//随机选择一个内部成员作为作者
			CmsUser user = this.userList.get((int)(Math.random()*(this.userList.size()-1))+0);
			cmsAcquisitionMng.saveContent(contentBean.getTitle(), contentBean.getContent(),contentBean.getContentList(),contentBean.getBrief(),titleImgPath,tagArr,imagesPath,imagesDesc,attachmentPaths,attachmentNames,attachmentFilenames,mediaPath,mediaType,attr,user,history,AcquisitionResultType.SUCCESS,temp,contentBean.getAcquId());
			cmsAcquisitionTempMng.save(temp);
			cmsAcquisitionHistoryMng.save(history);
		} catch (Exception e) {
			log.warn(null, e);
			handerResult(temp, history, null,AcquisitionResultType.UNKNOW);
		}
	}
	private void handerResult(CmsAcquisitionTemp temp,
			CmsAcquisitionHistory history, String title,
			AcquisitionResultType errorType) {
		handerResult(temp, history, title, errorType, false);
	}

	private void handerResult(CmsAcquisitionTemp temp,
			CmsAcquisitionHistory history, String title,
			AcquisitionResultType errorType, Boolean repeat) {
		temp.setDescription(errorType.name());
		temp.setTitle(title);
		cmsAcquisitionTempMng.save(temp);
		if (!repeat) {
			history.setTitle(title);
			history.setDescription(errorType.name());
			cmsAcquisitionHistoryMng.save(history);
		}
	}
	/**
	 * 取得媒体类型
	 * @param path 媒体全路径
	 * @return
	 */
	private String getMediaType(String path) {
		String ext = FilenameUtils.getExtension(path);
		if("swf".equals(ext.toLowerCase())){
			return "FLASH";
		}else if("flv".equals(ext.toLowerCase())){
			return "FLV";
		}else if("wm".equals(ext.toLowerCase())){
			return "WM";
		}else if("real".equals(ext.toLowerCase())){
			return "REAL";
		}else{
			return ext;
		}
	}
	/**
	 * 完成采集任务
	 * @param id 采集任务ID
	 */
	public void finished(Integer id) {
		cmsAcquisitionMng.end(id);
	}
	/**
	 * 检查采集任务
	 * 已经采集过 返回true
	 * 
	 * @param isTitle 是否根据标题判断,否则以URL
	 * @param value 值
	 */
	public synchronized boolean check(boolean isTitle,String value){
		return cmsAcquisitionHistoryMng.checkExistByProperties(isTitle, StringUtils.trim(value));
	}
	public List<String> getTagList(String str) {
		List<String> tagArr = new ArrayList<String>();
		try {
			Reader reader = new StringReader(str);
			//Analyzer analyzer = new StandardAnalyzer(Version.LUCENE_30);
			TokenStream stream = analyzer.tokenStream("", reader);
			TermAttribute termAtt = stream.addAttribute(TermAttribute.class);
			while (stream.incrementToken()) {
				if(StringUtils.isNotEmpty(termAtt.term()) && termAtt.term().length() >= 2){
					tagArr.add(termAtt.term());
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return tagArr;
	}
	
	public void initUserList() {
		Pagination pagination = cmsUserMng.getPage(null, null, null, 3, false, null, null, 1, 100);
		this.userList = (List<CmsUser>) pagination.getList();
	}
	private CmsAcquisitionTemp newTemp(String channelUrl, String contentUrl,
			Integer id, Float curr, Float total, CmsSite site) {
		CmsAcquisitionTemp temp = new CmsAcquisitionTemp();
		temp.setChannelUrl(channelUrl);
		temp.setContentUrl(contentUrl);
		temp.setSeq(id);
		NumberFormat nf = NumberFormat.getPercentInstance();
		String percent = nf.format(curr / total);
		temp.setPercent(Integer.parseInt(percent.substring(0,
				percent.length() - 1)));
		temp.setSite(site);
		return temp;
	}

	private CmsAcquisitionHistory newHistory(String channelUrl,
			String contentUrl, CmsAcquisition acqu) {
		CmsAcquisitionHistory history = new CmsAcquisitionHistory();
		history.setChannelUrl(channelUrl);
		history.setContentUrl(contentUrl);
		history.setAcquisition(acqu);
		return history;
	}
	
	  public static void main(String[] args) throws IOException {  
		  //Lucene Document的域名 
		    String fieldName = "干蒸花"; 
		     //检索内容 
		    String text = "IK Analyzer是一个结合词典分词干蒸和文法分词的中文分词开源工具包。它使用了全新的正向迭代最细粒度切分算法。"; 
		     
		    //实例化IKAnalyzer分词器 
		    Analyzer analyzer = new IKAnalyzer(); 
		     
		    Directory directory = null; 
		    IndexWriter iwriter = null; 
		    IndexSearcher isearcher = null; 
		    try { 
		      //建立内存索引对象 
		      directory = new RAMDirectory();    
		      iwriter = new IndexWriter(directory, analyzer, true , 
		IndexWriter.MaxFieldLength.LIMITED); 
		      Document doc = new Document(); 
		      doc.add(new Field(fieldName, text, Field.Store.YES, 
		Field.Index.ANALYZED)); 
		      iwriter.addDocument(doc); 
		      iwriter.close(); 
		       
		        //实例化搜索器    
		      isearcher = new IndexSearcher(directory);      
		      //在索引器中使用IKSimilarity相似度评估器 
		      isearcher.setSimilarity(new IKSimilarity()); 
		      
		      String keyword = "中文分词工具包"; 
		       
		      //使用IKQueryParser查询分析器构造Query对象 
		      Query query = IKQueryParser.parse(fieldName, keyword); 
		       
		      //搜索相似度最高的5条记录 
		      TopDocs topDocs = isearcher.search(query , 5); 
		      System.out.println("命中：" + topDocs.totalHits); 
		      //输出结果 
		      ScoreDoc[] scoreDocs = topDocs.scoreDocs; 
		      for (int i = 0; i < topDocs.totalHits; i++){ 
		        Document targetDoc = isearcher.doc(scoreDocs[i].doc); 
		        System.out.println("内容：" + targetDoc.toString()); 
		      }       
		       
		    } catch (CorruptIndexException e) { 
		      e.printStackTrace(); 
		    } catch (LockObtainFailedException e) { 
		      e.printStackTrace(); 
		    } catch (IOException e) { 
		      e.printStackTrace(); 
		    } finally{ 
		      if(isearcher != null){ 
		        try { 
		          isearcher.close(); 
		        } catch (IOException e) { 
		          e.printStackTrace(); 
		        } 
		      } 
		      if(directory != null){ 
		          try { 
		            directory.close(); 
		          } catch (IOException e) { 
		            e.printStackTrace(); 
		          } 
		        } 
		      } 
		    Reader reader = new StringReader(text);
		    TokenStream stream = analyzer.tokenStream("", reader);
			TermAttribute termAtt = stream.addAttribute(TermAttribute.class);
			while (stream.incrementToken()) {
				System.out.println("分词：" + termAtt.term()); 
				
			}
		    
	  }
}

