package org.javacoo.crawler.data;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.springframework.util.CollectionUtils;

/**
 * URL队列
 * @author javacoo
 * @since 2011-11-01
 * @param <Map<String, String>>
 */
public class UrlQueue {
	/**已访问URL集合*/
	private Set<Map<String, String>> visitedUrl = new HashSet<Map<String, String>>();
	/**待访问URL集合*/
	private Queue<Map<String, String>> unVisitedUrl = new BlockingQueue<Map<String, String>>();

	/**
	 * 获得 URL 队列
	 * @return
	 */
	public Queue<Map<String, String>> getUnVisitedUrl() {
		return unVisitedUrl;
	}
	/**
	 * 未访问的 URL 出队列
	 * @return
	 */
	public Map<String, String> unVisitedUrlDeQueue() {
		return unVisitedUrl.deQueue();
	}
	/**
	 * 保证每个 URL 只被访问一次
	 * @param url
	 */
	public void addUnVisitedUrl(Map<String, String> urlMap) {
		if (!CollectionUtils.isEmpty(urlMap) && !unVisitedUrl.contains(urlMap) && !visitedUrl.contains(urlMap)){
			unVisitedUrl.enQueue(urlMap);
		}
	}
	/**
	 * 判断是否为空
	 * @return
	 */
	public boolean isEmpty(){
		return unVisitedUrl.isEmpty();
	}
	/**
	 * 未访问URL数量
	 * @return
	 */
	public int getUnVisitedUrlNum(){
		return unVisitedUrl.getSize();
	}
	/**
	 * 添加到访问过的URL队列中
	 * @param urlMap
	 */
	public void addVisitedUrl(Map<String, String> urlMap){
		visitedUrl.add(urlMap);
	}
	/**
	 * 删除访问过的URL
	 * @param urlMap
	 */
	public void removeVisitedUrl(Map<String, String> urlMap){
		visitedUrl.remove(urlMap);
	}
	/**
	 * 已访问URL数量
	 * @return
	 */
	public int getVisitedUrlNum(){
		return visitedUrl.size();
	}
	/**
	 * 清空URL队列
	 */
	public void clear(){
		unVisitedUrl.clear();
		visitedUrl.clear();
	}
}
