package org.javacoo.crawler.filter;

import java.util.HashMap;
import java.util.Map;

import org.javacoo.crawler.constants.Constants;

/**
 * 过滤器接口实现类-抽象实现类
 * @author javacoo
 * @since 2011-11-10
 */
public abstract class AbstractFilter implements Filter{
	/**过滤器内容MAP*/
	protected Map<String,String> map = new HashMap<String,String>();
	/**过滤器名称*/
	protected String filterName;
	/**字段名称*/
	protected String fieldName;
	public AbstractFilter(){initialize();}
	/**
	 * <li>约定采集参数格式如下</li>
	 * <li>1，标签属性/值形式，如：class=articleList|tips,id=fxwb|fxMSN|fxMSN</li>
	 * <li>2，标签名称形式，如：div,p,span</li>
	 * <li>3，混合形式，如：class=articleList|tips,id=fxwb|fxMSN|fxMSN,div,p,span</li>
	 * @param fethAreaTagStr 要采集的区域标签字符串
	 * @param filterAreaTagStr 要过滤的区域标签字符串
	 */
	public AbstractFilter(String fethAreaTagStr, String filterAreaTagStr){
		initialize();
		setFilterMap(fethAreaTagStr,filterAreaTagStr);
	}

	protected abstract void initialize();
	
	public Map<String, String> getFilter() {
		return map;
	}

	public String getFilterName() {
		return this.filterName;
	}
	
	public String getFieldName() {
		return this.fieldName;
	}
	/**
	 * 设置过滤器
	 * @param fethAreaTagStr 要采集的区域标签字符串
	 * @param filterAreaTagStr 要过滤的区域标签字符串
	 */
	public void setFilter(String fethAreaTagStr, String filterAreaTagStr) {
		setFilterMap(fethAreaTagStr,filterAreaTagStr);
	}
	/**
	 * 设置过滤器map
	 * @param fethAreaTagStr 要采集的区域标签字符串
	 * @param filterAreaTagStr 要过滤的区域标签字符串
	 */
	private void setFilterMap(String fethAreaTagStr, String filterAreaTagStr){
		map.put(Constants.FETCH_AREA_KEY, fethAreaTagStr);
		map.put(Constants.FILTER_AREA_KEY, filterAreaTagStr);
	}
}
