package org.javacoo.crawler.processor.extractor;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.protocol.BasicHttpContext;
import org.springframework.util.CollectionUtils;

import org.javacoo.crawler.constants.Constants;
import org.javacoo.crawler.data.Task;
import org.javacoo.crawler.util.HttpGetHandler;

/**
 * 任务处理器接口-抽取内容资源实现类
 * @author javacoo
 * @since 2011-11-15
 */
public class ExtractorContentResourceProcessor extends Extractor{
	
	public ExtractorContentResourceProcessor() {
		super();
		setDefaultNextProcessor(new ExtractorFieldProcessor());
	}
	@SuppressWarnings("unchecked")
	@Override
	protected void extract(Task task) {
		log.info("=========抽取内容资源=========");
		Map<String,String> contentResMap = (Map<String,String>)task.getContentBean().getResMap().get(Constants.RES_IMAGES_MAP_KEY);
		if(task.getController().getCrawlScope().isExtractContentRes() && !CollectionUtils.isEmpty(contentResMap)){
			for(Iterator<String> it = contentResMap.keySet().iterator();it.hasNext();){
				HttpGet httpget = null;
				String key = it.next();
				String value = contentResMap.get(key);
				try {
					if(StringUtils.isNotEmpty(value)){
						if(value.contains("http://")){
							httpget = HttpGetHandler.getHttpGet(new URI(value.trim()));
						}else{
							httpget = HttpGetHandler.getHttpGet(new URI(task.getController().getCrawlScope().getRepairImageUrl()+value.trim()));
						}
						HttpResponse response = task.getController().getHttpClient().execute(httpget, new BasicHttpContext());
						HttpEntity entity = response.getEntity();
						if (entity != null) {
							task.getController().getCrawlScope().getFileHelper().saveFile(entity.getContent(), Constants.SYSTEM_ROOT_PATH +key);
						}
					}
				}  catch (IOException e) {
					e.printStackTrace();
				} catch (URISyntaxException e) {
					e.printStackTrace();
				}finally{
					if(null != httpget){
						httpget.abort();
					}
				}
			}
		}
		
	}
	

}
