package org.javacoo.crawler.util.parser;

import java.util.List;
import java.util.Map;

/**
 * HTML解析工具类接口
 * @author javacoo
 * @since 2011-10-31
 */
public interface ParseHtmlTool {
	/**
	 * 取得连接集合
	 * @param orginHtml 原始HTML
	 * @return 连接集合
	 */
	List<String> getUrlList( String orginHtml);
	/**
	 * 取得标题集合
	 * @param orginHtml 原始HTML
	 * @return 标题集合
	 */
	List<String> getTitleList(String orginHtml);
    /**
     * 取得指定区域的HTML内容
     * @return 指定区域的HTML内容
     */
	String getHtml(String orginHtml);
	/**
     * 取得指定区域的HTML内容
     * @return 指定区域的HTML内容
     */
	String getTagetHtml(String orginHtml);
	/**
     * 过滤指定区域的HTML内容
     * @return 过滤的HTML内容
     */
	String filterTagetHtml(String orginHtml);
	/**
	 * 取得无格式的HTML内容
	 * @param orginHtml 原始HTML内容
	 * @return 无格式的HTML内容
	 */
	String getPlainText(String orginHtml);
	/**
	 * 取得HTML内容中的资源
	 * @param orginHtml 原始HTML内容
	 * @param resMap 资源MAP
	 * @param  repairUrl 资源补全地址
	 * @return String
	 */
	String getHtmlResource(String orginHtml,Map<String,Object> resMap,String repairUrl);
	/**
	 * 取得连接标题Map集合，如果有标题图片则保存
	 * @param orginHtml 原始HTML
	 * @param savePath 标题图片保存路径
	 * @return 连接标题Map集合
	 */
    List<Map<String,String>> getUrlAndTitleMap(String orginHtml,String savePath);
	/**
	 * 替换HTML内容中资源链接地址为指定地址，并返回包含了替换后HTML，原始资源链接与替换后资源链接对照的MAP对象
	 * @param orginHtml 原始HTML内容
	 * @param savePath 资源保存路径
	 * @param resMap 资源MAP
	 * @return MAP
	 */
	void replaceHtmlResource(String orginHtml,String savePath,Map<String,Object> resMap);
	/**
	 * 替换内容中的超链接
	 * @param orginHtml 原始HTML内容
	 * @return 替换掉超链接后的HTML
	 */
	String replaceHtmlLink(String orginHtml);
	/**
	 * 取得原始HTML中指定条件的值并放在对应字段
	 * @param orginHtml 原始HTML
	 * @return 字段 值对照MAP
	 */
	Map<String,List<String>> getFieldValues(String orginHtml);
	/**
     * 取得指定HTML内容中的摘要信息
     * @param orginHtml 摘要信息
     * @return 指定区域的HTML内容
     */
	String getBrief(String orginHtml);
	/**
     * 取得指定HTML内容中的分页链接列表信息
     * @param orginHtml 原始HTML信息
     * @return 内容分页链接列表
     */
	List<String> getPaginationList(String orginHtml);
}
